package uploadsrcdir

import (
    "path/filepath"
    "slices"
    "os"
    "io/fs"
    "fmt"
    "strings"
    "gitea.armuli.eu/museum-digital/museum-digital-webdav-uploader/src/configloader"
)

var MetadataFileExts = []string{".xml", ".csv", ".json"}
var MediaFileExts = []string{".jpg", ".jpeg", ".png", ".gif", ".bmp", ".tif", ".tiff", ".pdf", ".mp3", ".mp4"}

// Lists all files in a folder that are of any of the expected file extensions.
func getFilesInDir(startPath string, fileExts []string) []os.FileInfo {

	allFiles := []os.FileInfo{}
	filepath.WalkDir(startPath, func(path string, entry fs.DirEntry, err error) error {

        if err != nil {
            return err
        }

        if slices.Contains(fileExts, filepath.Ext(entry.Name())) || slices.Contains(fileExts, strings.ToLower(filepath.Ext(entry.Name()))) {
            // add entry and it's size to dirMap
            file, err := entry.Info()
            if err != nil {
                fmt.Println("Error retrieving file info for:", err)
            } else {
                allFiles = append(allFiles, file)
            }
        }

        return nil

	})

    return allFiles

}

// Lists all files to upload in metadata directory
func ListUploadableMetadata(config configloader.MDWebDavUploaderConfig) []os.FileInfo {

    return getFilesInDir(config.UploadDir, MetadataFileExts);

}

// Lists all files to upload in metadata directory
func ListUploadableMedia(config configloader.MDWebDavUploaderConfig) []os.FileInfo {

    return getFilesInDir(config.UploadDir, MediaFileExts);

}

// Returns lists of uploadable folder contents grouped into a) metadata files and b) media files.
func GetUploadableFiles(config configloader.MDWebDavUploaderConfig) ([]string, []string) {

    metadataFiles := []string{}
    mediaFiles := []string{}

	filepath.WalkDir(config.UploadDir, func(path string, entry fs.DirEntry, err error) error {

        if err != nil {
            return err
        }

        if slices.Contains(MetadataFileExts, filepath.Ext(entry.Name())) || slices.Contains(MetadataFileExts, strings.ToLower(filepath.Ext(entry.Name()))) {
            metadataFiles = append(metadataFiles, path)
        }

        if slices.Contains(MediaFileExts, filepath.Ext(entry.Name())) || slices.Contains(MediaFileExts, strings.ToLower(filepath.Ext(entry.Name()))) {
            mediaFiles = append(mediaFiles, path)
        }

        return nil

	})

    return metadataFiles, mediaFiles

}
